"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.update = exports.timestampCondition = exports.updateExpression = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _dynamodb = _interopRequireDefault(require("../connectors/dynamodb"));

var _faults = require("./faults");

var _print = require("./print");

const updateExpression = Item => ({
  ExpressionAttributeNames: Object.keys(Item).map(attrName => ({
    [`#${attrName}`]: attrName
  })).reduce(_merge.default, {}),
  ExpressionAttributeValues: Object.keys(Item).map(attrName => ({
    [`:${attrName}`]: Item[attrName]
  })).reduce(_merge.default, {}),
  UpdateExpression: `SET ${Object.keys(Item).map(attrName => `#${attrName} = :${attrName}`).join(', ')}`,
  ReturnValues: 'ALL_NEW'
});

exports.updateExpression = updateExpression;

const timestampCondition = (fieldName = 'timestamp') => ({
  ConditionExpression: `attribute_not_exists(#${fieldName}) OR #${fieldName} < :${fieldName}`
});

exports.timestampCondition = timestampCondition;

const update = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.ENTITY_TABLE_NAME,
  updateRequestField = 'updateRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName
  });

  const invoke = uow => {
    const p = connector.update(uow[updateRequestField]).then(updateResponse => ({ ...uow,
      updateResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.update = update;